﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SqrxzEdit
{
    public partial class FormMain : Form
    {
        // consts
        public readonly int[,] tileChart = {{0,1,2,3,12,13,94,15,17,16,18,95,97,109,110,111},
                                                   {8,9,52,53,32,33,34,35,64,93,98,65,66,67,68,69},
                                                   {10,11,54,55,36,37,38,39,101,102,70,71,72,73,74,75},
                                                   {48,49,56,57,0,40,41,0,103,104,76,77,78,79,80,81},
                                                   {50,51,58,59,0,42,43,0,105,106,82,83,84,85,86,87},
                                                   {46,47,62,63,60,44,45,61,107,108,88,89,90,91,92,0},
                                                   {128,129,130,131,132,133,0,0,0,0,0,0,0,0,0,0}};
        const string progName = "Sqrxz World Editor";

        // vars
        Point tileStart, tileEnd;
        Point editPos = new Point(-1, -1);
        int contextCol;
        bool selectingTiles = false, editMouseFlag = false;
        Rectangle brush = new Rectangle(0, 0, 1, 1);
        World world;
        int level, editScrX;

        public FormMain()
        {
            InitializeComponent();
            world = new World();
            level = editScrX = 0;
            updateWindow();
            picEdit.Refresh();
        }

        private void picEdit_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int x = e.X / 32;
                int y = e.Y / 32;
                world.Levels[level].ApplyBrush(x + editScrX, y, brush);
                editMouseFlag = true;
                picEdit.Refresh();
            }
            else if (e.Button == MouseButtons.Right)
            {
                contextCol = (e.X / 32) + editScrX;
            }
        }

        private void picEdit_MouseLeave(object sender, EventArgs e)
        {
            editPos.X = editPos.Y = -1;
            picEdit.Refresh();
        }

        private void picEdit_MouseMove(object sender, MouseEventArgs e)
        {
            int x = editPos.X = e.X / 32;
            int y = editPos.Y = e.Y / 32;
            if (editMouseFlag)
            {
                if (x >= 0 && x <= 15 && y >= 0 && y <= 7)
                    world.Levels[level].ApplyBrush(x + editScrX, y, brush);
            }
            picEdit.Refresh();
        }

        private void picEdit_MouseUp(object sender, MouseEventArgs e)
        {
            editMouseFlag = false;
            picView.Refresh();
        }

        private void picEdit_Paint(object sender, PaintEventArgs e)
        {
            Bitmap render = world.Levels[level].LevelImage.Clone(new Rectangle(editScrX * 16, 0, 240, 128), world.Levels[level].LevelImage.PixelFormat);
            Bitmap renderX2 = new Bitmap(480, 256);
            using (Graphics g = Graphics.FromImage(renderX2))
            {
                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
                g.DrawImage(render, 0, 0, 481, 257);
            }
            e.Graphics.DrawImage(renderX2, 0, 0);

            if (editPos.X != -1 && editPos.Y != -1)
            {
                int x = editPos.X;
                int y = editPos.Y;
                Bitmap tileset = new Bitmap(picTiles.Image);
                Bitmap tileImage = tileset.Clone(new Rectangle(brush.X * 32, brush.Y * 32, brush.Width * 32, brush.Height * 32), tileset.PixelFormat);
                for (int bx = 0; bx < tileImage.Width; bx++)
                {
                    for (int by = 0; by < tileImage.Height; by++)
                    {
                        Color c = tileImage.GetPixel(bx, by);
                        tileImage.SetPixel(bx, by, Color.FromArgb(192, c));
                    }
                }
                e.Graphics.DrawImage(tileImage, x * 32, y * 32);
            }
        }

        private void picTiles_MouseDown(object sender, MouseEventArgs e)
        {
            tileStart.X = tileEnd.X = e.X / 32;
            tileStart.Y = tileEnd.Y = e.Y / 32;
            selectingTiles = true;
            picTiles.Refresh();
        }

        private void picTiles_MouseMove(object sender, MouseEventArgs e)
        {
            if (selectingTiles)
            {
                tileEnd.X = e.X / 32;
                tileEnd.Y = e.Y / 32;
                if (tileEnd.X > 15) { tileEnd.X = 15; }
                if (tileEnd.X < 0) { tileEnd.X = 0; }
                if (tileEnd.Y > 6) { tileEnd.Y = 6; }
                if (tileEnd.Y < 0) { tileEnd.Y = 0; }
                int x = Math.Min(tileStart.X, tileEnd.X);
                int y = Math.Min(tileStart.Y, tileEnd.Y);
                int w = Math.Abs(tileStart.X - tileEnd.X) + 1;
                int h = Math.Abs(tileStart.Y - tileEnd.Y) + 1;
                brush = new Rectangle(x, y, w, h);
                picTiles.Refresh();
            }
        }

        private void picTiles_MouseUp(object sender, MouseEventArgs e)
        {
            if (selectingTiles)
            {
                selectingTiles = false;
                int x = Math.Min(tileStart.X, tileEnd.X);
                int y = Math.Min(tileStart.Y, tileEnd.Y);
                int w = Math.Abs(tileStart.X - tileEnd.X) + 1;
                int h = Math.Abs(tileStart.Y - tileEnd.Y) + 1;
                brush = new Rectangle(x, y, w, h);
                picTiles.Refresh();
            }
        }

        private void picTiles_Paint(object sender, PaintEventArgs e)
        {
            Rectangle rect = new Rectangle(brush.X * 32, brush.Y * 32, brush.Width * 32, brush.Height * 32);
            e.Graphics.FillRectangle(new SolidBrush(Color.FromArgb(128, 255, 255, 255)), rect);
        }

        private void picView_MouseClick(object sender, MouseEventArgs e)
        {
            editScrX = (e.X / 16) + hScrollBar.Value - 7;
            if (editScrX < 0)
                editScrX = 0;
            else if ((editScrX + 15) >= world.Levels[level].Length)
                editScrX = world.Levels[level].Length - 15;
            updateWindow();
        }

        private void picView_Paint(object sender, PaintEventArgs e)
        {
            Bitmap render = world.Levels[level].LevelImage.Clone(new Rectangle(hScrollBar.Value * 16, 0, 1024, 128), world.Levels[level].LevelImage.PixelFormat);
            e.Graphics.DrawImage(render, 0, 0);

            int left = editScrX - hScrollBar.Value;
            Rectangle rect = new Rectangle(left * 16, 0, 15 * 16, 8 * 16);
            e.Graphics.FillRectangle(new SolidBrush(Color.FromArgb(64, 255, 255, 255)), rect);
        }

        private void numericTime_ValueChanged(object sender, EventArgs e)
        {
            world.Levels[level].Time = (int)numericTime.Value;
        }

        private void textName_TextChanged(object sender, EventArgs e)
        {
            world.Name = textName.Text;
        }

        private void textAuthor_TextChanged(object sender, EventArgs e)
        {
            world.Author = textAuthor.Text;
        }

        private void hScrollBar_Scroll(object sender, ScrollEventArgs e)
        {
            picView.Refresh();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog openFileWindow = new OpenFileDialog())
            {
                openFileWindow.Filter = "Sqrxz World Files (*.8xv *.8xp *.83p *.86s)|*.8xv;*.8xp;*.83p;*.86s";
                if (openFileWindow.ShowDialog() == DialogResult.OK)
                {
                    // open file and load contents to byte[]
                    BinaryReader reader = new BinaryReader(File.Open(openFileWindow.FileName, FileMode.Open));
                    string fileName = openFileWindow.SafeFileName;
                    List<byte> file = TIFile.ToList(reader, fileName.Substring(fileName.Length - 3));
                    reader.Close();

                    // verify Sqrxz compressed world
                    if (file[0] == 0x02 && file[1] == 0x01)
                    {
                        // load the world
                        world = new World(file);
                        world.Levels[0].Render();
                        world.FileName = fileName.Substring(0, fileName.Length - 4);
                        // reset some vars
                        level = 0;
                        hScrollBar.Value = 0;
                        editScrX = 0;
                        updateWindow();
                    }
                    else
                    {
                        MessageBox.Show("Not a valid Sqrxz world", "Error");
                    }
                }
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!world.Validate())
                return;
            using (SaveFileDialog saveFileWindow = new SaveFileDialog())
            {
                saveFileWindow.Filter = "Sqrxz World Files (*.8xv)|*.8xv";
                if (saveFileWindow.ShowDialog() == DialogResult.OK)
                {
                    BinaryWriter writer = new BinaryWriter(File.Open(saveFileWindow.FileName, FileMode.Create));
                    string fileName = Path.GetFileNameWithoutExtension(saveFileWindow.FileName);
                    world.FileName = fileName;
                    byte[] data = world.ToArray();
                    TIFile.Write(data, writer, world.FileName, "8XV");
                    writer.Close();
                    updateWindow();
                }
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void previousLevelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (level > 0)
            {
                level--;
                if (world.Levels[level].LevelImage == null)
                    world.Levels[level].Render();
                hScrollBar.Value = 0;
                editScrX = 0;
                updateWindow();
            }
        }

        private void nextLevelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (level < (world.NumLevels - 1))
            {
                level++;
                if (world.Levels[level].LevelImage == null)
                    world.Levels[level].Render();
                hScrollBar.Value = 0;
                editScrX = 0;
                updateWindow();
            }
        }

        private void deleteLevelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (world.NumLevels > 1)
            {
                if (MessageBox.Show("Delete this level?", "Confirm", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    world.NumLevels--;
                    world.Levels.RemoveAt(level);
                    if (level >= world.NumLevels)
                        level = world.NumLevels - 1;
                    updateWindow();
                }
            }
            else
            {
                MessageBox.Show("World must contain at least one level");
            }
        }

        private void addLevelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (world.NumLevels < 20)
            {
                world.NumLevels++;
                world.Levels.Add(new Level());
                updateWindow();
            }
            else
            {
                MessageBox.Show("World cannot contain more than 20 levels");
            }
        }

        private void moveLevelEarlierToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (level > 0)
            {
                world.SwapLevels(level - 1, level);
                level--;
                updateWindow();
            }
        }

        private void moveLevelLaterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (level < (world.NumLevels - 1))
            {
                world.SwapLevels(level, level + 1);
                level++;
                updateWindow();
            }
        }

        private void decreaseWidthToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (world.Levels[level].Length > 15)
            {
                world.Levels[level].Length--;
                updateWindow();
            }
        }

        private void increaseWidthToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (world.Levels[level].Length < 255)
            {
                world.Levels[level].Length++;
                updateWindow();
            }
        }

        private void insertColumnToolStripMenuItem_Click(object sender, EventArgs e)
        {
            world.Levels[level].InsertColumn(contextCol);
            updateWindow();
        }

        private void deleteColumnToolStripMenuItem_Click(object sender, EventArgs e)
        {
            world.Levels[level].DeleteColumn(contextCol);
            updateWindow();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormAbout about = new FormAbout();
            about.ShowDialog();
        }


        private void updateWindow()
        {
            if (world.Levels[level].LevelImage == null)
                world.Levels[level].Render();
            string fileName = world.FileName;
            if (fileName == "")
                fileName = "untitled";
            this.Text = progName + " - [" + fileName + "]";
            textName.Text = world.Name;
            textAuthor.Text = world.Author;
            textNumLevels.Text = world.NumLevels.ToString();
            textLevel.Text = (level + 1).ToString();
            textWidth.Text = world.Levels[level].Length.ToString();
            numericTime.Value = world.Levels[level].Time;
            hScrollBar.Maximum = Math.Max(world.Levels[level].Length - 64, 0);
            if ((editScrX + 15) >= world.Levels[level].Length)
                editScrX = world.Levels[level].Length - 15;
            picEdit.Refresh();
            picView.Refresh();
        }
    }
}
