﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SqrxzEdit
{
    static class TIFile
    {
        public static List<byte> ToList(BinaryReader reader, string fileType)
        {
            List<byte> file = new List<byte>();
            int len;

            switch (fileType.ToUpper())
            {
                case "83P":
                    reader.ReadBytes(8 + 3 + 42 + 2); // skip header
                    reader.ReadBytes(2 + 2 + 1 + 8 + 2); // skip data header
                    len = reader.ReadUInt16();
                    file = new List<byte>(reader.ReadBytes(len));
                    break;

                case "8XP":
                case "8XV":
                    reader.ReadBytes(8 + 3 + 42 + 2); // skip header
                    byte type = reader.ReadByte(); // either 0x0B or 0x0D
                    type -= 0x0B; // change it to either 0 or 2
                    reader.ReadBytes(1 + 2 + 1 + 8 + type + 2); // skip data header
                    len = reader.ReadUInt16();
                    file = new List<byte>(reader.ReadBytes(len));
                    break;
                
                case "86S":
                    reader.ReadBytes(8 + 3 + 42 + 2); // skip header
                    reader.ReadBytes(2 + 2 + 1); // skip first part of data header
                    int nameLen = reader.ReadByte(); // length of name
                    reader.ReadBytes(nameLen + 2);
                    len = reader.ReadUInt16();
                    file = new List<byte>(reader.ReadBytes(len));
                    break;
            }

            return file;
        }


        public static void Write(byte[] content, BinaryWriter writer, string name, string format)
        {
            if (format.ToUpper() == "8XV")
            {
                // start header with 8 & 3 byte signatures and comment
                writer.Write(NullString.ToList("**TI83F*", false).ToArray());
                writer.Write(new byte[] { 0x1A, 0x0A, 0x00 });
                writer.Write(NullString.ToList("Created by Sqrxz World Editor by JamesV   ", false).ToArray());

                // prepare the data section
                List<byte> data = new List<byte>();
                data.Add(0x0D);
                data.Add(0x00);
                ushort varlen = (ushort)(content.Length + 2);
                data.Add((byte)(varlen & 0xFF));
                data.Add((byte)(varlen >> 8));
                data.Add(0x15); // appvar object
                List<byte> name8 = NullString.ToList(name, false);
                while (name8.Count < 8)
                    name8.Add(0x00);
                data.AddRange(name8); // file name padded to 8-char length with null
                data.Add(0x00); // version
                data.Add(0x80); // archived
                data.Add((byte)(varlen & 0xFF));
                data.Add((byte)(varlen >> 8));
                ushort datlen = (ushort)content.Length;
                data.Add((byte)(datlen & 0xFF));
                data.Add((byte)(datlen >> 8));
                data.AddRange(content);

                // finish header by writing data section length
                datlen = (ushort)data.Count;
                writer.Write(datlen);

                // write data and calculate checksum
                ushort checksum = 0;
                foreach (byte b in data.ToArray())
                {
                    writer.Write(b);
                    checksum += b;
                }

                // write checksum
                writer.Write(checksum);
            }
        }
    }
}
