﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SqrxzEdit
{
    class World
    {
        public int NumLevels;
        public string FileName;
        public string Name;
        public string Author;
        public List<Level> Levels;

        
        public World()
        {
            NumLevels = 1;
            FileName = "";
            Name = "Untitled World";
            Author = "Unknown";
            Levels = new List<Level>();
            Levels.Add(new Level());
            Levels[0].Render();
        }


        public World(List<byte> file)
        {
            NumLevels = file[3];
            file.RemoveRange(0, 4); // remove header (word) and numLevels (word)
            Name = NullString.ReadString(file);
            file.RemoveRange(0, Name.Length + 2); // remove name and author width byte
            Author = NullString.ReadString(file);
            file.RemoveRange(0, Author.Length + 1); // remove author

            // initialise Level objects
            Levels = new List<Level>();

            // read level info
            for (int i = 0; i < NumLevels; i++)
            {
                int length = file[1];
                int time = file[2] << 8 | file[3];
                Levels.Add(new Level(length, time));
                file.RemoveRange(0, 4);
            }

            // remainder of file is a Huffman archive, convert it to byte[]
            byte[] archive = file.ToArray();

            // unpack level tile data and render levels
            for (int i = 0; i < NumLevels; i++)
                Levels[i].LoadTileData(Huffman.UnPack(archive, i));
        }


        public byte[] ToArray()
        {
            // put all levels into files to be packed
            List<byte[]> files = new List<byte[]>();
            foreach (Level l in Levels)
                files.Add(l.ToArray());

            // pack all levels into a Huffman archive
            byte[] archive = Huffman.Pack(files);

            // write world info
            List<byte> data = new List<byte>();
            data.Add(0x02);
            data.Add(0x01);
            data.Add(0x00);
            data.Add((byte)NumLevels);
            data.AddRange(NullString.ToList(Name, true));
            data.Add((byte)(4 * Author.Length));
            data.AddRange(NullString.ToList(Author, true));

            // write level headers
            foreach (Level l in Levels)
            {
                data.Add(0x00);
                data.Add((byte)l.Length);
                data.Add((byte)(l.Time >> 8));
                data.Add((byte)(l.Time & 0xFF));
            }

            // write Huffman archive
            data.AddRange(archive);

            return data.ToArray();
        }


        public void SwapLevels(int first, int second)
        {
            Level temp = Levels[first];
            Levels[first] = Levels[second];
            Levels[second] = temp;
        }


        public bool Validate()
        {
            if (Name.Length == 0 || Author.Length == 0)
            {
                MessageBox.Show("World must have a Name and Author", "Error");
                return false;
            }
            foreach (Level l in Levels)
            {
                if (!l.Validate())
                    return false;
            }
            return true;
        }
    }
}
